import time
from pynput import keyboard, mouse
import pyautogui
import os
import re
import keyboard as kb
import pandas as pd
import datetime

# Future work: 
#   Include PC sound, microphone, webcam
#   Make the same for smartphones
#   avoid adsminitrator permissions on ubuntu
#   The key identification for alt and other some keys will be changed from keyboard to keyboard 

#   Make easy installation for windows users using pip install 

class Collector:
    def __init__(self, base_folder='data/'):
        self.mouse = mouse.Controller()
        self.counter = 0
        self.lastscreen = None
        self.data = []
        self.moves = []
        self.movelistener = mouse.Listener(on_move=self.on_move)
        self.movesflag = False
        self.base_folder = base_folder
        self.sample_folder = self.create_incremented_folder(self.base_folder) + '/' 
        
    # Monitoring
    def start(self):
        print("Waiting for activation with ESC key ...")
        kb.wait('esc')

        print("Monitoring is working ...")
        self.movelistener.start()  
        self.date = datetime.datetime.now()
        self.running = True
        self.start_time = time.perf_counter()
        self.lastscreen = (time.perf_counter() - self.start_time, pyautogui.screenshot())
        with mouse.Listener(on_click=self.on_click, on_scroll=self.on_scroll), \
             keyboard.Listener(on_press=self.on_press, on_release=self.on_release):
            while self.running:
                self.lastscreen = (time.perf_counter() - self.start_time, pyautogui.screenshot())
        print("Monitoring was ended ...")

        data_df = pd.DataFrame(self.data, columns =['timestamp', 'img_path', 'px', 'py', 'event', 'trajectory'])
        data_df.to_csv(self.sample_folder + f'rawdata_{self.date}.csv')
        self.movelistener.stop()

    # Mouse events
    def on_move(self, x, y):
        if self.movesflag == True:
            self.moves.append((time.perf_counter() - self.start_time, x, y)) 
        else:
            self.moves=[]

    def on_click(self, px, py, button, pressed):
        if pressed:
            self.movesflag = True
            self.savedata(px, py, event=f'pressed {button}')
        else:
            self.movesflag = False
            self.savedata(px, py, event=f'released {button}', trajectory=self.moves)

    def on_scroll(self, px, py, dx, dy):
        if dy == 1:
            scroll = 'scroll_down' 
        elif dy == -1:
            scroll = 'scroll_up'
        else:
            scroll = 'what?_scroll'
        self.savedata(px, py, event=scroll)
    
    # Keyboard events 
    def on_press(self, key):
        if key == keyboard.Key.esc: 
            self.running = False
        key = str(key).strip("'")
        px, py = self.mouse.position
        self.savedata(px, py, event=f'pressed {key}')
        
    def on_release(self, key):
        key = str(key).strip("'")
        px, py = self.mouse.position
        self.savedata(px, py, event=f'released {key}')

    def savedata(self, px, py, event, trajectory=None):
        timestamp = time.perf_counter() - self.start_time
        t, img = self.lastscreen
        img_path = self.sample_folder + 'screen{:010}_{}.jpg'.format(self.counter, t)
        self.data.append((timestamp, img_path, px, py, event, trajectory))
        img.save(img_path)
        self.counter += 1

    # Initialize folder to save data
    def create_incremented_folder(self, path):
        if not os.path.exists(path):
            os.makedirs(path)
        # Find the highest number
        dirs = [d for d in os.listdir(path) if os.path.isdir(os.path.join(path, d))]
        highest_num = 0
        for dir_name in dirs:
            match = re.match(r"sample(\d+)", dir_name)
            if match:
                highest_num = max(highest_num, int(match.group(1)))
        # Create new folder
        new_folder_name = f"sample{highest_num + 1}"
        new_folder_path = os.path.join(path, new_folder_name)
        os.makedirs(new_folder_path)
        return new_folder_path

    



import pyautogui
import time
import numpy as np
from PIL import Image, ImageDraw, ImageFont
from pathlib import Path

# current limitations: 
# No drag actions, 
# Not pressed two keys o more at the same time
# pcontroller is prepared for all events

# Verify the current initial screen is compatable with the recorded initial screen. 
# Allow compounding tasks for big ones.   
class Replayer():
    def __init__(self, sample, data_dir):
        self.sample = sample
        self.data_dir = data_dir
 
    def preprocessing_actions(self): 
        actions = self.sample[self.sample['event'].map(lambda x: 'pressed' in x)].copy()
        actions['event'] = actions['event'].map(lambda x: x.replace('pressed ',''))
        delays = actions['timestamp'][1:].values - actions['timestamp'][0:-1].values
        delays = np.append(delays,0.0)
        actions['delay'] = delays
        actions = actions.reset_index(drop=True)
        self.actions = actions 

    def pcontroller(self, action, position, delay): 
        # Mouse
        if "Button." in action:
            if action == 'Button.left': 
                pyautogui.click(x=position[0], y=position[1])
            elif action == 'Button.right': 
                pyautogui.click(button='right', x=position[0], y=position[1])
        elif "scroll_" in action:
            if action == 'scroll_down': 
                pyautogui.scroll(-1, x=position[0], y=position[1])
            elif action == 'scroll_up': 
                pyautogui.scroll(1, x=position[0], y=position[1])
        # Keyboard
        elif "Key." in action: 
            if action == 'Key.enter':
                pyautogui.moveTo(position[0],position[1])
                pyautogui.press('enter')
            elif action == 'Key.esc':
                pyautogui.moveTo(position[0],position[1])
                pyautogui.press('esc')
        # Write with keyboard
        elif len(action) == 1 and action.isalpha():
            pyautogui.moveTo(position[0],position[1])
            pyautogui.write(action)
        else:
            print(f'This action {action} does not exist')
        time.sleep(delay)

    def viz_actions(self, index, action, screen_flag):
        """ Tool for debugging actions """
        if index == 0:
            self.mfont = ImageFont.truetype("arial.ttf", 40)
            self.kfont = ImageFont.truetype("arial.ttf", 80)
            self.size = 10
            self.viz_dir = self.data_dir + 'viz/'
            Path(self.viz_dir).mkdir(parents=True, exist_ok=True)

        if screen_flag == True: 
            img = pyautogui.screenshot()
        else:
            img = Image.open(self.data_dir + action['img_path'])
        draw = ImageDraw.Draw(img)
        px = action['px']
        py = action['py']
        if 'Button.' in action['event']: 
            draw.ellipse([px-self.size/2,py-self.size/2,px+self.size//2,py+self.size//2], fill="red")
            draw.point((px,py), fill='yellow')
            draw.text((px,py), str(index), fill='red', font=self.mfont)
        else:
            draw.text((1920//2 - 100,1080//2), action['event'], fill='red', font=self.kfont)
        img.save(self.viz_dir + 'step{:04}.jpg'.format(index))
    
    def execute(self, viz=False, screen_flag=True):
        self.preprocessing_actions()
        time.sleep(2)
        for index, action in self.actions.iterrows():
            if viz == True: 
                self.viz_actions(index, action, screen_flag)
            self.pcontroller(action['event'], (action['px'], action['py']), action['delay'])
        pyautogui.alert('The execution has been terminated.')


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pccloner',
    version='0.0.8',
    description= "Clone your repetitive PC tasks",
    py_modules = ['pccloner.pcdata', 'pccloner.pctask'],
    packages = find_packages(),
    package_dir = {'':'src'},
    
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    
    install_requires = ['pynput >= 1.7.6',
                        'pyautogui >= 0.9.54',
                        'keyboard >= 0.13.5',
                        'pandas >= 2.1.4',
                       ],
    
    url="https://github.com/ezamorag/pccloner",
    author="Erik Zamora",
    author_email="ezamora1981@gmail.com",
    
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.10',
)

